#!/usr/bin/env python2.5

"""Compare found matches with seed-matches

This program expects a match file and a seed-match file.
Every seed-match must be found in the match file with at most the
number of implanted seed errors in the surroundings of at most 200bp.
If a seed-match was not found the seed-match and the matches for the
corresponding read are written to stderr. At the end the tuple:
#Not-Found #Found #1-Less #2-Less #3-Less #4-Less #5+ -Less
is written to stdout, where #Not-Found is the number of seed-matches
not found in the surroundings of 200bp. #i-Less is the number of seed-
matches that were found with i more errors than implanted and #Found
is the number of matches that were correctly found with the number
of implemented or less errors.

"""

__author__ = 'David Weese <david.weese@fu-berlin.de>, Sandro Andreotti <andreott@fu-berlin.de>'

import sys
import subprocess
import os
import tempfile
import time
import csv

def main(args):
  if (len(args) < 2):
    print args[0] + " <matches> <seedmatches>"
    return 0

  testfile = open(args[1], 'r')
  goldfile = open(args[2], 'r')
  
  golddict = {}
  testdict = {}
  
  tolerance = 200

  reader = csv.reader(goldfile)
  for row in reader:
    golddict[row[0]] = golddict.get(row[0],[]) + [[int(row[1]),int(row[2]), int(row[3])]]

  reader = csv.reader(testfile)
  for row in reader:
    testdict[row[0]] = testdict.get(row[0],[]) + [[int(row[1]),int(row[2]), int(row[3])]]
  
  matches_all = 0
  matches_notfound = 0
  matches_found = [0] * 100
  for read in golddict:
    for match in golddict[read]:
      matches_all += 1
      best_found = 1000
      for found in testdict.get(read,[]):
        if ((abs(found[0] - match[0]) < tolerance) and (abs(found[1] - match[1]) < tolerance) and (best_found > found[2])):
          best_found = found[2];
      
      delta = best_found - match[2]
      if (delta < 0):
        delta = 0

      if (best_found == 1000):
      	matches_notfound += 1
      else:
        matches_found[delta] += 1

      if (delta != 0):
        print >> sys.stderr, 'Match:       ', match, read
        print >> sys.stderr, 'FoundInstead:', testdict.get(read,[])
  print matches_notfound, '\t', matches_found[0], '\t', matches_found[1], '\t', matches_found[2], '\t', matches_found[3], '\t', matches_found[4], '\t', sum(matches_found[5:])
  return 0


if __name__ == '__main__':
  sys.exit(main(sys.argv))

